var thrift = require('thrift'),
    ttransport = require('thrift/transport'),
    ThriftHive = require('gen-nodejs/ThriftHive')
    ResultSet = require('./result_set')

var futureConnection = function(config) {
  return function(cb) {
    var connection = thrift.createConnection(config.server, config.port || 10000, {transport: ttransport.TBufferedTransport, timeout: config.timeout || 1000});
    var client = thrift.createClient(ThriftHive, connection);
    cb(client, connection);
  };
};

var hiveClient = function(futureConnection) {
  return {
    fetch: function(query, cb) {
      futureConnection(function(client, connection) {
        client.execute(query, function(err){
          if(err) return cb(true, err);
          client.getSchema(function(err, schema) {
            if (err) return cb(true, err);
            client.fetchAll(function(err, data){
              if (err) return cb(true, err);
              cb(null, ResultSet.create(data, schema));
              connection.end();
            });
          });
        });
      })
    },
    
    fetchInBatch: function(batchSize, query, cb) {
      futureConnection(function(client, connection) {
        client.execute(query, function(err){
          if (err) return cb(true, err);
          client.getSchema(function(err, schema) {
            if (err) return cb(true, err);
            var fetchBatch = function() {
              client.fetchN(batchSize, function(err, data){
                if (err) return cb(true, err);
                if(data.length > 0) {
                  cb(null, ResultSet.create(data, schema));
                  process.nextTick(fetchBatch);
                }
                else connection.end();
              });
            };
            fetchBatch();
          });
        });
      })
    },
    
    execute: function(query, cb){
      futureConnection(function(client, connection) {
        client.execute(query, function(err){
          if (err) return cb(true, err);
          cb(null, null);
          connection.end();
        });
      });
    },
  };
};

exports.for = function(config) {
  return hiveClient(futureConnection(config));
};
